// -*- C++ -*-
/*!
 * @file  MiddleRobotCtrl.cpp
 * @brief RH中位系動作制御モジュール
 * @date $Date$
 *
 * $Id$
 */

#include "MiddleRobotCtrl.h"
#include "MiddleRobotCtrl_mat.cpp"
#include "MiddleRobotCtrl_mat_data.cpp"
#include "rt_look.cpp"
#include "rt_look1d.cpp"
#include "rtGetInf.cpp"
#include "rtGetNaN.cpp"
#include "rt_nonfinite.cpp"
#include "rt_pow_snf.cpp"

static double Arm_in[5];
static double Obj_in[7];
static double Ord_in;

// Module specification
// <rtc-template block="module_spec">
static const char* middlerobotctrl_spec[] =
  {
    "implementation_id", "MiddleRobotCtrl",
    "type_name",         "MiddleRobotCtrl",
    "description",       "RH中位系動作制御モジュール",
    "version",           "1.0.0",
    "vendor",            "TokyoMetropolitanUniversity",
    "category",          "Calcuration",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.conf_test", "1",
    // Widget
    "conf.__widget__.conf_test", "text",
    // Constraints
    ""
  };
// </rtc-template>



/*!
 * @brief constructor
 * @param manager Maneger Object
 */
MiddleRobotCtrl::MiddleRobotCtrl(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Obj_inIn("ObjectInfo", m_Obj_in),
    m_Arm_inIn("ArmInfo", m_Arm_in),
    m_Mobile_inIn("MobileInfo", m_Mobile_in),
    m_Order_inIn("OrderInfo", m_Order_in),
    m_Arm_outOut("ArmCtrl", m_Arm_out),
    m_Mobile_outOut("MobileCtrl", m_Mobile_out)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
MiddleRobotCtrl::~MiddleRobotCtrl()
{
}



RTC::ReturnCode_t MiddleRobotCtrl::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("ObjectInfo", m_Obj_inIn);
  addInPort("ArmInfo", m_Arm_inIn);
  addInPort("MobileInfo", m_Mobile_inIn);
  addInPort("OrderInfo", m_Order_inIn);
  
  // Set OutPort buffer
  addOutPort("ArmCtrl", m_Arm_outOut);
  addOutPort("MobileCtrl", m_Mobile_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t MiddleRobotCtrl::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MiddleRobotCtrl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MiddleRobotCtrl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t MiddleRobotCtrl::onActivated(RTC::UniqueId ec_id)
{
  MiddleRobotCtrl_mat_initialize();
  std::cout<<"initialize"<<std::endl;
  
  
  int i;
  for(i=0;i<5;i++)  Arm_in[i]=0.0;
  for(i=0;i<7;i++)  Obj_in[i]=0.0;
  Ord_in = 0.0;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t MiddleRobotCtrl::onDeactivated(RTC::UniqueId ec_id)
{
  MiddleRobotCtrl_mat_terminate();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t MiddleRobotCtrl::onExecute(RTC::UniqueId ec_id)
{
  int i;
  static int k; 
  /////////////////  input  ////////////////////////////////////
  if( m_Obj_inIn.isNew()){
     m_Obj_in.data.length(7);
     m_Obj_inIn.read();
     
     std::cout<<"Object_IN ";
     for(i=0;i<7;i++){
     	 Obj_in[i]=m_Obj_in.data[i];
       std::cout<<Obj_in[i]<<", ";
     }
     std::cout<<std::endl;
  }
  if( m_Arm_inIn.isNew()){
     m_Arm_in.data.length(5);
     m_Arm_inIn.read();
     std::cout<<"Arm_IN ";
     for(i=0;i<5;i++){
     	 Arm_in[i]=m_Arm_in.data[i];
     	 std::cout<<Arm_in[i]<<", ";
     }
     std::cout<<std::endl;
  }else{
  	 Arm_in[0]=0;
  }
  if( m_Order_inIn.isNew()){
     m_Order_inIn.read();
     Ord_in = m_Order_in.data;
     std::cout<<"Order_IN "<<Ord_in<<std::endl;
  }
//  input //
  for(i=0;i<7;i++){
     MiddleRobotCtrl_mat_U.ObjectInfo[i] = Obj_in[i];
  }
  for(i=0;i<5;i++){
     MiddleRobotCtrl_mat_U.ArmInfo[i] = Arm_in[i];
  }
  MiddleRobotCtrl_mat_U.OrderInfo = Ord_in;

  //////////////////////////////////////////////////////////////
  MiddleRobotCtrl_mat_step(); // execute//
//  std::cout<<"after step"<<std::endl;

  if(k>1000){
  	std::cout<<"RAC_ID: "<<MiddleRobotCtrl_mat_Y.RAC_ID<<std::endl;
//	  std::cout<<" Mode_ID: "<<(int)MiddleRobotCtrl_mat_Y.Mode_ID;
//  	std::cout<<"OK_flag: "<<Arm_in[0]<<std::endl;
  	k=0;
  }else{
  	k++;
  }

  
  /////////////////  output  ///////////////////////////////////
  switch((int)MiddleRobotCtrl_mat_Y.Mode_ID){
  case 0:
	break;
  case 1:
		m_Arm_out.data.length(13);
		m_Arm_out.data[0] = 1;
		for(i=1;i<13;i++){
			m_Arm_out.data[i] = MiddleRobotCtrl_mat_Y.TransMatrix[i-1];
		}
		m_Arm_outOut.write();
		std::cout<<"Arm_OUT [";
		      for(i=0;i<13;i++) std::cout<< m_Arm_out.data[i] <<", ";
		      std::cout<<"]"<<std::endl;
	break;

  case 2:
		m_Arm_out.data.length(2);
		m_Arm_out.data[0] = 2;
		m_Arm_out.data[1] = MiddleRobotCtrl_mat_Y.Gripper;
		m_Arm_outOut.write();
		std::cout<<"Arm_OUT [";
		      for(i=0;i<2;i++) std::cout<< m_Arm_out.data[i] <<", ";
		      std::cout<<"]"<<std::endl;
	break;

  case 3:
		m_Arm_out.data.length(1);
		m_Arm_out.data[0] = 3;
		m_Arm_outOut.write();
		std::cout<<"Arm_OUT ["<<m_Arm_out.data[0]<<"]"<<std::endl;
	break;
	
  default:
		m_Arm_out.data.length(1);
		m_Arm_out.data[0] = 3;
		m_Arm_outOut.write();
		std::cout<<"モードエラー"<< (int)MiddleRobotCtrl_mat_Y.Mode_ID <<std::endl;
	break; 
 }

  //////////////////////////////////////////////////////////////
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t MiddleRobotCtrl::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t MiddleRobotCtrl::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t MiddleRobotCtrl::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MiddleRobotCtrl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MiddleRobotCtrl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void MiddleRobotCtrlInit(RTC::Manager* manager)
  {
    coil::Properties profile(middlerobotctrl_spec);
    manager->registerFactory(profile,
                             RTC::Create<MiddleRobotCtrl>,
                             RTC::Delete<MiddleRobotCtrl>);
  }
  
};


