// -*- C++ -*-
/*!
 * @file  ArmCtrl.cpp * @brief arm command accepter component * $Date$ 
 *
 * $Id$ 
 */
#include "ArmCtrl.h"

// Module specification
// <rtc-template block="module_spec">
static const char* armctrl_spec[] =
  {
    "implementation_id", "ArmCtrl",
    "type_name",         "ArmCtrl",
    "description",       "arm command accepter component",
    "version",           "1.0",
    "vendor",            "Junichiro Ooga, TOSHIBA",
    "category",          "RobotController",
    "activity_type",     "SPORADIC",
    "kind",              "DataFowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

ArmCtrl::ArmCtrl(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_cmdIn("cmd", m_cmd),
    m_dataOut("data", m_data)

    // </rtc-template>
{
}

ArmCtrl::~ArmCtrl()
{
}


RTC::ReturnCode_t ArmCtrl::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("cmd", m_cmdIn);

  // Set OutPort buffer
  addOutPort("data", m_dataOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  // </rtc-template>
  cout << "finish onInitialize !" << endl;

  return RTC::RTC_OK;
}



RTC::ReturnCode_t ArmCtrl::onFinalize()
{
  cout << "finish onFinalize !" << endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ArmCtrl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ArmCtrl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t ArmCtrl::onActivated(RTC::UniqueId ec_id)
{
  // m_data.data.length(2);
  // for(int i=0; i<2; i++){
  //   m_data.data[i] = 0;
  // }
  // m_dataOut.write();

  cout << "finish onActivated !" << endl;
  return RTC::RTC_OK;
}

RTC::ReturnCode_t ArmCtrl::onDeactivated(RTC::UniqueId ec_id)
{

  cout << "finish onDeActivated !" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ArmCtrl::onExecute(RTC::UniqueId ec_id)
{

  if(!m_cmdIn.isNew()){
    return RTC::RTC_OK;
  }

  m_cmdIn.read();  // read Inport

  if(m_cmd.data[0]<0){
    cout << "[Error]" << endl;
    m_data.data.length(2);
    for(int i=0; i<2; i++){
      m_data.data[i] = 0;
    }
    m_dataOut.write();
  }else{

    switch((int)m_cmd.data[0]){
    case 1:
      cout << "Cartesian coordinate position control [ ";
      cout << m_cmd.data[4] << ", " << m_cmd.data[8] << ", " << m_cmd.data[12] << "]" << endl;
      
      usleep(5000000);
      m_data.data.length(5);
      //      m_data.data[0] = m_cmd.data[4];   // x
      //      m_data.data[1] = m_cmd.data[8];   // y
      //      m_data.data[2] = m_cmd.data[12];  // z
      m_data.data[0] = 1;  // 
      m_data.data[1] = 10;  //
      m_data.data[2] = 20;  //
      m_data.data[3] = 30;  //
      m_data.data[4] = 1.0; // hand width
             
      // m_data.data[2] = 1.0;    // debug
      // m_data.data[3] = 0.0;   // hand
      // m_data.data[4] = 1;     // complete flag
      m_dataOut.write();
      cout << "Finish cartesian coordinate position control" << endl;
      cout << "[" << m_data.data[0] << ", " << m_data.data[1] << "]" << endl;
      break;
    case 2:
      cout << "Hand position control: Open width=" << m_cmd.data[1] << "[%]" << endl;
      if(m_cmd.data[1]<0 || m_cmd.data[1]>100){
	cout << "[Error] Illegal parameter: out of range !" << endl;
	m_data.data[0] = -1;
	
	m_dataOut.write();
	return RTC::RTC_OK;
      }
      usleep(2000000);
 //     m_data.data.length(2);
      //      m_data.data[0] = 100;   // x
      //      m_data.data[1] = 100;   // y
      //      m_data.data[2] = 100;   // z
 //     m_data.data[0] = 20;   // 
 //     m_data.data[1] = 1;   //       
      // m_data.data[2] = 1;   // debug
      // m_data.data[3] = m_cmd.data[1];   // hand
      // m_data.data[4] = 1;     // complete flag
      m_data.data[0] = 2;  // 
      m_data.data[1] = 10;  // 
      m_data.data[2] = 20;  //
      m_data.data[3] = 30;  //
      m_data.data[4] = 100.0; // hand width

      m_dataOut.write();      
      cout << "Finish cartesian coordinate position control" << endl;
      cout << "[" << m_data.data[0] << ", " << m_data.data[1] << "]" << endl;
      break;
    default:
      cout << "[Error]Undefined command number " << m_cmd.data[0] << endl;
      break;
    }
    
  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ArmCtrl::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ArmCtrl::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ArmCtrl::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ArmCtrl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ArmCtrl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void ArmCtrlInit(RTC::Manager* manager)
  {
    coil::Properties profile(armctrl_spec);
    manager->registerFactory(profile,
                             RTC::Create<ArmCtrl>,
                             RTC::Delete<ArmCtrl>);
  }
  
};



